<?php
/**
 * Emoji.
 * Note: Database must use 'utf8mb4' to allow emojis. 'utf8' is not enough.
 *
 * @package Flat101\Security\Model\Block
 */

namespace Flat101\Security\Model\Block;

use Carbon_Fields\Field\Field;
use Flat101\Security\Model\Entity\Block;

/**
 * Emoji class.
 */
class EmojiBlock extends Block {

	/**
	 * Block constructor.
	 */
	public function __construct() {
		parent::__construct(
			esc_html__( 'Emojis', 'flat101-security' ),
			'emoji',
			array(
				'disabled' => true,
			)
		);
	}

	// PHPDoc (inherited).
	// phpcs:ignore Squiz.Commenting.FunctionComment.Missing
	public function get_fields() {
		return array(

			Field::make( 'checkbox', $this->get_option_name( 'disabled' ), __( 'Disabled', 'flat101-security' ) )
			->set_default_value( $this->default_settings['disabled'] ),

		);
	}

	// PHPDoc (inherited).
	// phpcs:ignore Squiz.Commenting.FunctionComment.Missing
	public function run() {
		if ( $this->settings['disabled'] ) {
			$this->disabled();
		}
	}

	/**
	 * Disable the emoji's
	 */
	public function disabled() {
		remove_action( 'wp_head', 'print_emoji_detection_script', 7 );
		remove_action( 'admin_print_scripts', 'print_emoji_detection_script' );
		remove_action( 'wp_print_styles', 'print_emoji_styles' );
		remove_action( 'admin_print_styles', 'print_emoji_styles' );
		remove_filter( 'the_content_feed', 'wp_staticize_emoji' );
		remove_filter( 'comment_text_rss', 'wp_staticize_emoji' );
		remove_filter( 'wp_mail', 'wp_staticize_emoji_for_email' );
		add_filter( 'tiny_mce_plugins', array( $this, 'disable_emojis_tinymce' ) );
		add_filter( 'wp_resource_hints', array( $this, 'disable_emojis_remove_dns_prefetch' ), 10, 2 );
	}

	/**
	 * Filter function used to remove the tinymce emoji plugin.
	 *
	 * @param array $plugins An array of default TinyMCE plugins.
	 * @return array Difference betwen the two arrays
	 */
	public function disable_emojis_tinymce( $plugins ) {
		if ( is_array( $plugins ) ) {
			return array_diff( $plugins, array( 'wpemoji' ) );
		} else {
			return array();
		}
	}

	/**
	 * Remove emoji CDN hostname from DNS prefetching hints.
	 *
	 * @param array  $urls URLs to print for resource hints.
	 * @param string $relation_type The relation type the URLs are printed for.
	 * @return array Difference betwen the two arrays.
	 */
	public function disable_emojis_remove_dns_prefetch( $urls, $relation_type ) {
		if ( 'dns-prefetch' === $relation_type ) {
			// This filter is documented in wp-includes/formatting.php.
			// phpcs:ignore WordPress.NamingConventions.PrefixAllGlobals.NonPrefixedHooknameFound
			$emoji_svg_url = apply_filters( 'emoji_svg_url', 'https://s.w.org/images/core/emoji/14.0.0/svg/' );

			$urls = array_diff( $urls, array( $emoji_svg_url ) );
		}

		return $urls;
	}
}
